#ifndef PIECE_SEEN
#define PIECE_SEEN

// PieceShapes hold the 2-d bit-vectors defining shapes of pieces.  If shape
// is a PieceShape, and shape[x][y] is true, then that piece shape contains a
// 1x1 square at location (x, y).  (0, 0) is the top left of the piece.

const int MAXPIECEDIM   = 6;                // pieces can be at most 6x6

typedef bool PieceShape[MAXPIECEDIM][MAXPIECEDIM];

class InputSpec;

// A Piece represents an instance of a piece, with a location.

class Piece
{
    int     top, left;          // where is the upper left corner of this piece?
    int     shapeIndex;         // what shape is it?  (index into shapes array in
                                // input spec)
    
public:

    static  InputSpec *spec;        // the input specifications
    
    
            Piece(int whichShape, int l, int t);
            Piece(Piece &piece);
    
    bool    solved();
    bool    isGoal();
    bool    occupied(int x, int y);
    bool    canSlide(int dir, class Configuration *config);
    void    shift(int dir);
    
    void    print();
    
    bool    sameAs(Piece *piece);
};


// A PieceList is a linked list of Piece pointers.

struct PieceList
{
    Piece       *piece;
    PieceList   *next;

                PieceList(Piece *p, PieceList *pl);
                PieceList(PieceList *pl);
                        
    void        deleteContents();
    
    void        print();

    Piece       *index(int pieceNum);
};

#endif
